/*
 * Copyright (C) 2016-2020 Kano Computing Ltd.
 * License: http://www.gnu.org/licenses/gpl-2.0.txt GNU General Public License v2
 */
import { Output } from './output.js';
import { join } from '../util/path.js';
export const RESOURCE_CACHE_RESOLUTION_MULTIPLIER = 2;
export class Resource {
    constructor() {
        this.cache = new Map();
        this.legacyIdMap = new Map();
        this.default = '';
        this.categories = {};
        this.all = [];
        this.allCategorised = [];
    }
    addCategory(id, label) {
        if (this.categories[id]) {
            console.warn('Category already exists');
            return;
        }
        this.categories[id] = { id, label, resources: {} };
    }
    add(category, id, label, path) {
        if (!this.categories[category]) {
            console.warn('Category does not exist');
            return;
        }
        if (this.categories[category].resources[id]) {
            console.warn('A sticker with this name has already been created');
            return;
        }
        this.categories[category].resources[id] = { id, label, path: path };
    }
    setDefault(id) {
        this.default = id;
    }
    resolve(path) {
        const ASSET_URL_PREFIX_KEY = 'sticker:base-url';
        const prefix = Output.config.get(ASSET_URL_PREFIX_KEY, '/assets/part/stickers');
        return encodeURI(join(prefix, path));
    }
    addLegacyIdMap(map) {
        this.legacyIdMap = map;
    }
    get resourceSet() {
        if (this.all.length > 0) {
            return this.all;
        }
        Object.values(this.categories).forEach(category => {
            Object.values(category.resources).forEach(sticker => {
                this.all.push(sticker);
            });
        });
        return this.all;
    }
    get categorisedResource() {
        if (this.allCategorised.length > 0) {
            return this.allCategorised;
        }
        Object.values(this.categories).forEach(category => {
            let categoryObject = {
                id: category.id,
                label: category.label,
                resources: [],
            };
            Object.values(category.resources).forEach(sticker => {
                const stickerObject = { id: sticker.id, label: sticker.label, src: this.getUrl(sticker.id) };
                categoryObject.resources.push(stickerObject);
            });
            this.allCategorised.push(categoryObject);
        });
        return this.allCategorised;
    }
    get categoryEnum() {
        return Object.values(this.categories).map(category => [category.label, category.id]);
    }
    cacheValue(id, type = 'image') {
        if (type !== 'image') {
            console.warn('cached item type not recognised');
            return;
        }
        return this.cache.get(id);
    }
    rasteriseImage(img) {
        const canvas = document.createElement('canvas');
        /* Normalise stickers to fit a 150x150 bounding box */
        if ((img.width !== 150 && img.height !== 150) || (img.width > 150 || img.height > 150)) {
            if (img.width >= img.height) {
                img.height = Math.round((150 / img.width) * img.height);
                img.width = 150;
            }
            else {
                img.width = Math.round((150 / img.height) * img.width);
                img.height = 150;
            }
        }
        canvas.width = img.width * RESOURCE_CACHE_RESOLUTION_MULTIPLIER;
        canvas.height = img.height * RESOURCE_CACHE_RESOLUTION_MULTIPLIER;
        const ctx = canvas.getContext('2d');
        if (ctx) {
            ctx.drawImage(img, 0, 0, canvas.width, canvas.height);
        }
        return canvas;
    }
    load(resource) {
        return new Promise((resolve, reject) => {
            const img = new Image();
            const started = Date.now();
            img.crossOrigin = "Anonymous";
            const onLoad = () => {
                this.cache.set(resource.id, this.rasteriseImage(img));
                resolve();
                img.removeEventListener('load', onLoad);
                img.removeEventListener('error', onError);
            };
            const onError = () => {
                reject();
                img.removeEventListener('load', onLoad);
                img.removeEventListener('error', onError);
            };
            img.addEventListener('load', onLoad);
            img.addEventListener('error', onError);
            img.src = this.resolve(resource.path);
        });
    }
    getRandom() {
        return this.resourceSet[Math.floor(Math.random() * this.resourceSet.length)].id;
    }
    getRandomFrom(id) {
        if (!this.categories[id]) {
            console.warn('Category does not exist');
            return this.default;
        }
        else {
            const randomKey = Object.keys(this.categories[id].resources)[Math.floor(Math.random() * Object.keys(this.categories[id].resources).length)];
            return this.categories[id].resources[randomKey].id;
        }
    }
    getUrl(id) {
        if (!id) {
            console.warn(`No sticker provided`);
            return '';
        }
        let selectedSticker = this.resourceSet.find(sticker => sticker.id === id);
        if (!selectedSticker) {
            selectedSticker = this.resourceSet.find(sticker => sticker.id === this.default);
        }
        if (!selectedSticker) {
            console.warn(`Sticker ${id} does not exist`);
            return '';
        }
        return this.resolve(selectedSticker.path);
    }
    getLabel(id) {
        const selectedSticker = this.resourceSet.find(sticker => sticker.id === id);
        if (!selectedSticker) {
            console.warn('Sticker does not exist');
            return;
        }
        return this.resolve(selectedSticker.label);
    }
}
export class Resources {
    constructor() {
        this.resources = new Map();
        this.resources.set('stickers', new Resource());
    }
    get(id) {
        return this.resources.get(id);
    }
    load(progress) {
        let total = 0;
        this.resources.forEach((resource) => {
            total += resource.resourceSet.length;
        });
        let completed = 0;
        const promises = [];
        this.resources.forEach((resource) => {
            resource.resourceSet.forEach((r) => {
                const p = resource.load(r)
                    .then(() => {
                    completed += 1;
                    progress(completed / total);
                }).catch(() => {
                    /* Report progress even for failed images */
                    completed += 1;
                    progress(completed / total);
                });
                promises.push(p);
            });
        });
        return Promise.all(promises);
    }
}
//# sourceMappingURL=resources.js.map