import '@kano/kwc-auth/dist/kwc-auth-example.js';
import { loadMessages, _ } from '@kano/i18n/index.js';
import { configs } from './config/config.js';
import * as channelOs from './lib/os-channel.js';
import * as channelPc from './lib/pc-channel.js';
import { extractEnv, getUrlParams } from './lib/util.js';
import { Actions } from './lib/actions.js';
import { flowDefinition } from './lib/flow.js';

let channel;
// Check type of machine
if (window.Windows) {
    channel = channelPc;
} else {
    // We don't have a specific implementation so use the generic OS one
    channel = channelOs;
}

const DEFAULT_LOCALE = 'en-us';
const SUPPORTED_LOCALES = [DEFAULT_LOCALE, 'ja-jp'];
function getLocale() {
    const lang = navigator.language.toLowerCase().slice(0, 2);
    for (const l of SUPPORTED_LOCALES) {
        if (lang === l.toLowerCase().slice(0, 2)) {
            return l;
        }
    }
    return SUPPORTED_LOCALES[0];
}

const ENV = extractEnv();
const config = configs[ENV];
const actions = new Actions(config.apiUrl);
const params = getUrlParams();
const urlLang = params && params.lang ? params.lang : false;
const locale = urlLang || getLocale();

function done() {
    if ('redirect_uri' in params) {
        window.location.replace(params.redirect_uri);
    } else {
        channel.quit();
    }
}

loadMessages(`/locale/${locale}.json`)
    .then(() => {
        // Authentication component
        const auth = document.createElement('kwc-auth-example');
        auth.addEventListener('view-changed', (e) => {
            channel.notifyViewChange(e.detail.id);
        });
        auth.style.backgroundColor = '#fff';
        auth.actions = actions;
        auth.flowDefinition = flowDefinition(_('BACK', 'Back'));
        auth.locale = locale;
        document.body.appendChild(auth);

        /**
         * Simples routing you can think of
         */
        switch (window.location.hash) {
            case '#signup':
                auth.changeTemplate('username');
                break;
            case '#login':
            default:
                auth.changeTemplate('login');
                break;
        }

        // Store the signup payload. It will be used once the user finalise their account
        let payload = null;

        // When the user effectively signs-up, keep track of the session
        actions.onDidSignup((session) => {
            payload = session;
        });

        // The user finished their registration and decided to start playing
        actions.onDidFinish(() => {
            if (!payload) {
                return;
            }
            channel.sendUserData(
                payload.user.id,
                payload.user.username,
                payload.user.email,
                payload.token,
                payload.duration,
                payload.user.attributes.consent,
                'signup'
            // TODO: Show an error message if SSO could not be set.
            ).finally(() => {
                done();
            });
        });

        // User succesfully logged in
        actions.onDidLogin((session) => {
            channel.sendUserData(
                session.user.id,
                session.user.username,
                session.user.email,
                session.token,
                session.duration,
                session.user.attributes.consent,
                'login'
            // TODO: Show an error message if SSO could not be set.
            ).finally(() => {
                done();
            });
        });

        // User exited auth dialog
        actions.onDidExit(() => {
            done();
        });

        // Potential params that can be passed to the auth login component
        if (params) {
            if ('allowExit' in params && params.allowExit === "true") {
                auth.allowExit = true;
            }
        }
    });
