import { Disposables, subscribeDOM, EventEmitter } from '@kano/common/index.js';

class AuthUI {
    constructor(config) {
        this.config = this._configOverrides(config);

        this._root = document.createElement('iframe');
        this.subscriptions = new Disposables();
        this._promise = null;

        this._onDidChangeView = new EventEmitter();

        subscribeDOM(window, 'message', this._onMessage, this, this.subscriptions);

        this.subscriptions.push(this._onDidChangeView);
    }

    get onDidChangeView() { return this._onDidChangeView.event; }

    dispose() {
        this.cancel();
        this.subscriptions.dispose();

        if (this._root && this._root.parentNode) {
            this._root.parentNode.removeChild(this._root);
        }
        this._root = null;
    }

    getDomNode() {
        return this._root;
    }

    _onMessage(e) {
        if (!e.data || !e.data.m) {
            return;
        }

        switch (e.data.m) {
        case 'auth-callback':
            if (this._promise) {
                this._promise.resolve(e.data.a);
                this._promise = null;
            }
            break;
        case 'change-view': {
            this._onDidChangeView.fire(e.data.a);
            break;
        }
        case 'quit':
            if (this._promise) {
                this._promise.reject('Quit');
                this._promise = null;
            }
            break;
        default:
            throw new Error('Unknown message');
        }
    }

    _loadIframe(url) {
        if (!this._root.parentNode) {
            throw new Error('Unable to load iframe: not attached to DOM.');
        }

        if (this._promise) {
            throw new Error('Authentication already in progress.');
        }

        return new Promise((resolve, reject) => {
            this._root.setAttribute('src', url);
            this._promise = { resolve, reject };
        });
    }

    _configOverrides(config) {
        let configOverrides = {};
        let signupUrl = config.AUTH_SIGNUP_URL ? new URL(config.AUTH_SIGNUP_URL) : false;
        let loginUrl = config.AUTH_LOGIN_URL ? new URL(config.AUTH_LOGIN_URL) : false;

        // Locale overrides
        if (config.LOCALE && typeof config.LOCALE === 'string' && config.LOCALE !== '') {
            if (signupUrl) signupUrl.searchParams.append('lang', config.LOCALE);
            if (loginUrl) loginUrl.searchParams.append('lang', config.LOCALE);
        }

        // Profile overrides
        switch (config.PROFILE) {
            case 'world':
                // Allow exit of signup flow only on kano world
                if (signupUrl) signupUrl.searchParams.append('allowExit', true);
                break;
            default:
                break;
        }

        if (signupUrl) configOverrides.AUTH_SIGNUP_URL = signupUrl.toString();
        if (loginUrl) configOverrides.AUTH_LOGIN_URL = loginUrl.toString();
        return Object.assign(config, configOverrides);
    }

    requestLogin() {
        return this._loadIframe(this.config.AUTH_LOGIN_URL);
    }

    requestSignup() {
        return this._loadIframe(this.config.AUTH_SIGNUP_URL);
    }

    cancel() {
        if (this.authInProgress()) {
            this._promise.reject('Aborted');
            this._promise = null;
        }
    }

    authInProgress() {
        return !!this._promise;
    }
}

export default {
    createUI(config) {
        return new AuthUI(config);
    },
};
