import { AccountClient, UserClient } from '@kano/api-client/index.js';
import { EventEmitter } from '@kano/common/index.js';
import { _ } from '@kano/i18n/index.js';

export class Actions {
    constructor(worldUrl) {
        this.account = new AccountClient({ url: worldUrl });
        this.user = new UserClient({ url: worldUrl });

        this._onDidLogin = new EventEmitter();
        this._onDidSignup = new EventEmitter();
        this._onDidFinish = new EventEmitter();
        this._onDidExit = new EventEmitter();
    }

    get onDidLogin() { return this._onDidLogin.event; }
    get onDidSignup() { return this._onDidSignup.event; }
    get onDidFinish() { return this._onDidFinish.event; }
    get onDidExit() { return this._onDidExit.event; }

    login(form) {
        return this.account.login(form.username, form.password)
            .then((s) => {
                return this.user.me(s.token)
                    .then((user) => {
                        this.token = s.token;
                        this.duration = s.duration;
                        this.userMe = user;
                        if (this.attributesInvalid(user)) {
                            return user;
                        }
                        this._onDidLogin.fire({ duration: s.duration, token: s.token, user });
                        return user;
                    });
            })
            .catch((e)=> {
                if (e.status === 401) {
                    // Username is incorrect
                    throw new Error(_('USERNAME_OR_PASSWORD_INCORRECT', 'Username or password incorrect'));
                }
                // Server error
                throw new Error(_('SOMETHING_WENT_WRONG', 'Something went wrong, please try again later.'));
            });
    }
    register(form) {
        return this.account.register(form)
            .then((s) => {
                return this.user.me(s.token)
                    .then((user) => {
                        this._onDidSignup.fire({ duration: s.duration, token: s.token, user });
                    });
            });
    }
    updateEmail(email) {
        return this.account.updateEmail(email, this.token)
            .then(() => {
                this._onDidLogin.fire({ duration: this.duration, token: this.token, user: this.userMe });
            })
            .catch((e)=> {
                throw e.status;
            });
    }
    updateUsername(username) {
        return this.account.updateUsername(username, this.token)
            .then(() => {
                if (this.userMe.attributes.email !== 'invalid' && this.userMe.attributes.email !== 'removed') {
                    this._onDidLogin.fire({ duration: this.duration, token: this.token, user: this.userMe });
                }
            })
            .catch((e)=> {
                throw e.status;
            });
    }
    getAuthenticatedUser() {
        return new Promise((resolve) => resolve(user));
    }
    checkUsernameAvailability(username) {
        return this.account.checkUsernameAvailability(username)
            .then((available) => {
                if (!available) {
                    return Promise.reject(new Error(_('USERNAME_ALREADY_TAKEN', 'Username already taken.')));
                }
            })
            .catch((e) => {
                if (!e.headers) {
                    throw e;
                }
                const errorCode = e.headers.get('X-Error-Code');
                // Didn't pass PII check
                if (errorCode === '400.12') {
                    return Promise.reject(new Error(_('USERNAME_INVALID', 'The username is invalid. Try a different username or check our community guidelines for more information.')));
                }
                throw e;
            });
    }
    forgotPassword(username) {
        return this.account.forgotPassword({ username });
    }
    forgotUsername(email) {
        return this.account.forgotUsername({ email });
    }
    forgotEmail(email) {
        console.log(email);
        return new Promise((resolve) => resolve());
    }
    resendEmail(userId) {
        console.log(userId);
        return new Promise((resolve) => resolve());
    }
    finish() {
        this._onDidFinish.fire();
        return Promise.resolve();
    }
    exit() {
        this._onDidExit.fire();
        return Promise.resolve();
    }

    attributesInvalid(user) {
        return user.attributes && (user.attributes.email === 'invalid' || user.attributes.email === 'removed' || user.attributes.username === 'invalid')
    }
};
