/**
 * \file pc-channel.js
 *
 * \copyright Copyright (C) 2020 Kano Computing Ltd.
 *
 *  Implementation web channel interface for Windows
 */


export const sendUserData = (id, username, email, token, duration, consent, method) => {
    return new Promise(function(resolve, reject) {
        if (!('Windows' in window)) {
            reject(new Error("Windows APIs not available"));
            return;
        }
        if (!('KanoHubServiceClient' in window && 'connectAsync' in KanoHubServiceClient && 'cacheLoginAsync' in KanoHubServiceClient)) {
            reject(new Error("KanoHubServiceClient is not available"));
            return;
        }
        if (!('LoginData' in window && 'userId' in LoginData && 'username' in LoginData && 'token' in LoginData && 'duration' in LoginData && 'consent' in LoginData)) {
            reject(new Error("LoginData is not available"));
            return;
        }

        KanoHubServiceClient.connectAsync()
        .then(
            function onComplete(status) {
                if (status != Windows.ApplicationModel.AppService.AppServiceConnectionStatus.success) {
                    reject(new Error("Could not connect to the service"));
                    return;
                }

                // Note: LoginData is an instance of the Data Contract object as encapsulated in
                // the KanoWorldContracts Windows Runtime Component. It establishes the data
                // format that the client is expecting. Any changes to the data structure will
                // require a mandatory update to Kano Hub and its clients while extensions will not.

                LoginData.userId = id;                  // string
                LoginData.username = username;          // string
                LoginData.token = token;                // string
                LoginData.duration = Number(duration);  // long
                LoginData.consent = consent;            // boolean

                // https://docs.microsoft.com/en-us/microsoft-edge/windows-runtime/using-windows-runtime-asynchronous-methods

                KanoHubServiceClient.cacheLoginAsync(LoginData)
                .then(
                    function onComplete(successful) {
                        successful ? resolve() : reject(new Error("cacheLoginAsync was not successful"));
                        return;
                    },
                    function onError(err) {
                        reject(new Error("Failed to cache login session with Kano Hub: " + err));
                        return;
                    }
                );
            },
            function onError(err) {
                reject(new Error("Failed to connect to the app service: " + err));
                return;
            }
        );
    });
};

export const quit = () => {
    if (!('KanoHubServiceClient' in window && 'finaliseLogin' in KanoHubServiceClient)) {
        throw "KanoHubServiceClient is not available";
    }
    KanoHubServiceClient.finaliseLogin();
};

export const notifyViewChange = (view) => {

};
